@@ -1995,9 +1995,7 @@ const char* g_pubsuffix[]={
 "aso.kumamoto.jp",
 "choyo.kumamoto.jp",
 "gyokuto.kumamoto.jp",
-"hitoyoshi.kumamoto.jp",
 "kamiamakusa.kumamoto.jp",
-"kashima.kumamoto.jp",
 "kikuchi.kumamoto.jp",
 "kumamoto.kumamoto.jp",
 "mashiki.kumamoto.jp",
@@ -3651,6 +3649,14 @@ const char* g_pubsuffix[]={
 "edu.my",
 "mil.my",
 "name.my",
+"ac.mz",
+"adv.mz",
+"co.mz",
+"edu.mz",
+"gov.mz",
+"mil.mz",
+"net.mz",
+"org.mz",
 "info.na",
 "pro.na",
 "name.na",
@@ -3689,20 +3695,20 @@ const char* g_pubsuffix[]={
 "net.ng",
 "org.ng",
 "sch.ng",
-"com.ni",
-"gob.ni",
-"edu.ni",
-"org.ni",
-"nom.ni",
-"net.ni",
-"mil.ni",
-"co.ni",
-"biz.ni",
-"web.ni",
-"int.ni",
 "ac.ni",
+"biz.ni",
+"co.ni",
+"com.ni",
+"edu.ni",
+"gob.ni",
 "in.ni",
 "info.ni",
+"int.ni",
+"mil.ni",
+"net.ni",
+"nom.ni",
+"org.ni",
+"web.ni",
 "bv.nl",
 "fhs.no",
 "vgs.no",
@@ -4644,133 +4650,9 @@ const char* g_pubsuffix[]={
 "in.rs",
 "org.rs",
 "ac.ru",
-"com.ru",
 "edu.ru",
-"int.ru",
-"net.ru",
-"org.ru",
-"pp.ru",
-"adygeya.ru",
-"altai.ru",
-"amur.ru",
-"arkhangelsk.ru",
-"astrakhan.ru",
-"bashkiria.ru",
-"belgorod.ru",
-"bir.ru",
-"bryansk.ru",
-"buryatia.ru",
-"cbg.ru",
-"chel.ru",
-"chelyabinsk.ru",
-"chita.ru",
-"chukotka.ru",
-"chuvashia.ru",
-"dagestan.ru",
-"dudinka.ru",
-"e-burg.ru",
-"grozny.ru",
-"irkutsk.ru",
-"ivanovo.ru",
-"izhevsk.ru",
-"jar.ru",
-"joshkar-ola.ru",
-"kalmykia.ru",
-"kaluga.ru",
-"kamchatka.ru",
-"karelia.ru",
-"kazan.ru",
-"kchr.ru",
-"kemerovo.ru",
-"khabarovsk.ru",
-"khakassia.ru",
-"khv.ru",
-"kirov.ru",
-"koenig.ru",
-"komi.ru",
-"kostroma.ru",
-"krasnoyarsk.ru",
-"kuban.ru",
-"kurgan.ru",
-"kursk.ru",
-"lipetsk.ru",
-"magadan.ru",
-"mari.ru",
-"mari-el.ru",
-"marine.ru",
-"mordovia.ru",
-"msk.ru",
-"murmansk.ru",
-"nalchik.ru",
-"nnov.ru",
-"nov.ru",
-"novosibirsk.ru",
-"nsk.ru",
-"omsk.ru",
-"orenburg.ru",
-"oryol.ru",
-"palana.ru",
-"penza.ru",
-"perm.ru",
-"ptz.ru",
-"rnd.ru",
-"ryazan.ru",
-"sakhalin.ru",
-"samara.ru",
-"saratov.ru",
-"simbirsk.ru",
-"smolensk.ru",
-"spb.ru",
-"stavropol.ru",
-"stv.ru",
-"surgut.ru",
-"tambov.ru",
-"tatarstan.ru",
-"tom.ru",
-"tomsk.ru",
-"tsaritsyn.ru",
-"tsk.ru",
-"tula.ru",
-"tuva.ru",
-"tver.ru",
-"tyumen.ru",
-"udm.ru",
-"udmurtia.ru",
-"ulan-ude.ru",
-"vladikavkaz.ru",
-"vladimir.ru",
-"vladivostok.ru",
-"volgograd.ru",
-"vologda.ru",
-"voronezh.ru",
-"vrn.ru",
-"vyatka.ru",
-"yakutia.ru",
-"yamal.ru",
-"yaroslavl.ru",
-"yekaterinburg.ru",
-"yuzhno-sakhalinsk.ru",
-"amursk.ru",
-"baikal.ru",
-"cmw.ru",
-"fareast.ru",
-"jamal.ru",
-"kms.ru",
-"k-uralsk.ru",
-"kustanai.ru",
-"kuzbass.ru",
-"mytis.ru",
-"nakhodka.ru",
-"nkz.ru",
-"norilsk.ru",
-"oskol.ru",
-"pyatigorsk.ru",
-"rubtsovsk.ru",
-"snz.ru",
-"syzran.ru",
-"vdonsk.ru",
-"zgrad.ru",
 "gov.ru",
+"int.ru",
 "mil.ru",
 "test.ru",
 "gov.rw",
@@ -5319,7 +5201,6 @@ const char* g_pubsuffix[]={
 "lib.co.us",
 "lib.ct.us",
 "lib.dc.us",
-"lib.de.us",
 "lib.fl.us",
 "lib.ga.us",
 "lib.gu.us",
@@ -5456,51 +5337,68 @@ const char* g_pubsuffix[]={
 "sch.zm",
 "beep.pl",
 "cloudfront.net",
-"compute.amazonaws.com",
-"ap-northeast-1.compute.amazonaws.com",
-"ap-northeast-2.compute.amazonaws.com",
-"ap-southeast-1.compute.amazonaws.com",
-"ap-southeast-2.compute.amazonaws.com",
-"eu-central-1.compute.amazonaws.com",
-"eu-west-1.compute.amazonaws.com",
-"sa-east-1.compute.amazonaws.com",
-"us-gov-west-1.compute.amazonaws.com",
-"us-west-1.compute.amazonaws.com",
-"us-west-2.compute.amazonaws.com",
-"compute-1.amazonaws.com",
-"z-1.compute-1.amazonaws.com",
-"z-2.compute-1.amazonaws.com",
 "us-east-1.amazonaws.com",
-"compute.amazonaws.com.cn",
-"cn-north-1.compute.amazonaws.com.cn",
-"elasticbeanstalk.com",
-"elb.amazonaws.com",
-"s3.amazonaws.com",
+"elasticbeanstalk.cn-north-1.amazonaws.com.cn",
 "s3-ap-northeast-1.amazonaws.com",
 "s3-ap-northeast-2.amazonaws.com",
+"s3-ap-south-1.amazonaws.com",
 "s3-ap-southeast-1.amazonaws.com",
 "s3-ap-southeast-2.amazonaws.com",
+"s3-ca-central-1.amazonaws.com",
 "s3-eu-central-1.amazonaws.com",
 "s3-eu-west-1.amazonaws.com",
 "s3-external-1.amazonaws.com",
-"s3-external-2.amazonaws.com",
 "s3-fips-us-gov-west-1.amazonaws.com",
 "s3-sa-east-1.amazonaws.com",
 "s3-us-gov-west-1.amazonaws.com",
+"s3-us-east-2.amazonaws.com",
 "s3-us-west-1.amazonaws.com",
 "s3-us-west-2.amazonaws.com",
 "s3.ap-northeast-2.amazonaws.com",
+"s3.ap-south-1.amazonaws.com",
 "s3.cn-north-1.amazonaws.com.cn",
+"s3.ca-central-1.amazonaws.com",
 "s3.eu-central-1.amazonaws.com",
+"s3.us-east-2.amazonaws.com",
+"s3.dualstack.ap-northeast-1.amazonaws.com",
+"s3.dualstack.ap-northeast-2.amazonaws.com",
+"s3.dualstack.ap-south-1.amazonaws.com",
+"s3.dualstack.ap-southeast-1.amazonaws.com",
+"s3.dualstack.ap-southeast-2.amazonaws.com",
+"s3.dualstack.ca-central-1.amazonaws.com",
+"s3.dualstack.eu-central-1.amazonaws.com",
+"s3.dualstack.eu-west-1.amazonaws.com",
+"s3.dualstack.sa-east-1.amazonaws.com",
+"s3.dualstack.us-east-1.amazonaws.com",
+"s3.dualstack.us-east-2.amazonaws.com",
+"s3-website-us-east-1.amazonaws.com",
+"s3-website-us-west-1.amazonaws.com",
+"s3-website-us-west-2.amazonaws.com",
+"s3-website-ap-northeast-1.amazonaws.com",
+"s3-website-ap-southeast-1.amazonaws.com",
+"s3-website-ap-southeast-2.amazonaws.com",
+"s3-website-eu-west-1.amazonaws.com",
+"s3-website-sa-east-1.amazonaws.com",
+"s3-website.ap-northeast-2.amazonaws.com",
+"s3-website.ap-south-1.amazonaws.com",
+"s3-website.ca-central-1.amazonaws.com",
+"s3-website.eu-central-1.amazonaws.com",
+"s3-website.us-east-2.amazonaws.com",
+"t3l3p0rt.net",
+"tele.amune.org",
 "on-aptible.com",
+"user.party.eus",
 "pimienta.org",
 "poivron.org",
 "potager.org",
 "sweetpepper.org",
+"myasustor.com",
 "myfritz.net",
 "backplaneapp.io",
 "betainabox.com",
+"bnr.la",
 "boxfuse.io",
+"browsersafetymark.io",
 "mycd.eu",
 "ae.org",
 "ar.com",
@@ -5548,8 +5446,28 @@ const char* g_pubsuffix[]={
 "r.cdn77.net",
 "rsc.cdn77.org",
 "ssl.origin.cdn77-secure.org",
+"cloudns.asia",
+"cloudns.biz",
+"cloudns.club",
+"cloudns.cc",
+"cloudns.eu",
+"cloudns.in",
+"cloudns.info",
+"cloudns.org",
+"cloudns.pro",
+"cloudns.pw",
+"cloudns.us",
 "co.nl",
 "co.no",
+"dyn.cosidns.de",
+"dynamisches-dns.de",
+"dnsupdater.de",
+"internet-dns.de",
+"l-o-g-i-n.de",
+"dynamic-dns.info",
+"feste-ip.net",
+"knx-server.net",
+"static-access.net",
 "realm.cz",
 "cupcake.is",
 "cyon.link",
@@ -5848,8 +5766,19 @@ const char* g_pubsuffix[]={
 "webhop.org",
 "worse-than.tv",
 "writesthisblog.com",
+"ddnss.de",
+"dyn.ddnss.de",
+"dyndns.ddnss.de",
+"dyndns1.de",
+"dyn-ip24.de",
+"home-webserver.de",
+"dyn.home-webserver.de",
+"myhome-server.de",
+"ddnss.org",
 "dynv6.net",
 "e4.cz",
+"enonic.io",
+"customer.enonic.io",
 "eu.org",
 "al.eu.org",
 "asso.eu.org",
@@ -5925,13 +5854,18 @@ const char* g_pubsuffix[]={
 "fbxos.fr",
 "freebox-os.fr",
 "freeboxos.fr",
+"myfusion.cloud",
+"futurehosting.at",
+"futuremailing.at",
 "service.gov.uk",
 "github.io",
 "githubusercontent.com",
 "githubcloud.com",
 "gist.githubcloud.com",
 "gitlab.io",
-"ro.com",
+"homeoffice.gov.uk",
+"ro.im",
+"shop.ro",
 "goip.de",
 "appspot.com",
 "blogspot.ae",
@@ -6013,6 +5947,7 @@ const char* g_pubsuffix[]={
 "googleapis.com",
 "googlecode.com",
 "pagespeedmobilizer.com",
+"publishproxy.com",
 "withgoogle.com",
 "withyoutube.com",
 "hashbang.sh",
@@ -6023,6 +5958,38 @@ const char* g_pubsuffix[]={
 "iki.fi",
 "biz.at",
 "info.at",
+"ac.leg.br",
+"al.leg.br",
+"am.leg.br",
+"ap.leg.br",
+"ba.leg.br",
+"ce.leg.br",
+"df.leg.br",
+"es.leg.br",
+"go.leg.br",
+"ma.leg.br",
+"mg.leg.br",
+"ms.leg.br",
+"mt.leg.br",
+"pa.leg.br",
+"pb.leg.br",
+"pe.leg.br",
+"pi.leg.br",
+"pr.leg.br",
+"rj.leg.br",
+"rn.leg.br",
+"ro.leg.br",
+"rr.leg.br",
+"rs.leg.br",
+"sc.leg.br",
+"se.leg.br",
+"sp.leg.br",
+"to.leg.br",
+"js.org",
+"keymachine.de",
+"knightpoint.systems",
+"co.krd",
+"edu.krd",
 "meteorapp.com",
 "eu.meteorapp.com",
 "co.pl",
@@ -6122,6 +6089,7 @@ const char* g_pubsuffix[]={
 "zapto.org",
 "nyc.mn",
 "nid.io",
+"opencraft.hosting",
 "operaunite.com",
 "outsystemscloud.com",
 "ownprovider.com",
@@ -6137,28 +6105,54 @@ const char* g_pubsuffix[]={
 "pantheonsite.io",
 "gotpantheon.com",
 "mypep.link",
+"on-web.fr",
 "xen.prgmr.com",
 "priv.at",
+"protonet.io",
 "chirurgiens-dentistes-en-france.fr",
 "qa2.com",
+"dev-myqnapcloud.com",
+"alpha-myqnapcloud.com",
+"myqnapcloud.com",
 "rackmaze.com",
 "rackmaze.net",
 "rhcloud.com",
 "hzc.io",
+"wellbeingzone.eu",
+"ptplus.fit",
+"wellbeingzone.co.uk",
 "sandcats.io",
 "logoip.de",
 "logoip.com",
+"firewall-gateway.com",
+"firewall-gateway.de",
+"my-gateway.de",
+"my-router.de",
+"spdns.de",
+"spdns.eu",
+"firewall-gateway.net",
+"my-firewall.org",
+"myfirewall.org",
+"spdns.org",
 "biz.ua",
 "co.ua",
 "pp.ua",
+"shiftedit.io",
 "myshopblocks.com",
+"1kapp.com",
+"appchizi.com",
+"applinzi.com",
 "sinaapp.com",
 "vipsinaapp.com",
-"1kapp.com",
 "bounty-full.com",
 "alpha.bounty-full.com",
 "beta.bounty-full.com",
+"static.land",
+"dev.static.land",
+"sites.static.land",
+"apps.lair.io",
 "spacekit.io",
+"stackspace.space",
 "diskstation.me",
 "dscloud.biz",
 "dscloud.me",
@@ -6172,6 +6166,7 @@ const char* g_pubsuffix[]={
 "i234.me",
 "myds.me",
 "synology.me",
+"taifun-dns.de",
 "gda.pl",
 "gdansk.pl",
 "gdynia.pl",
@@ -6180,14 +6175,41 @@ const char* g_pubsuffix[]={
 "bloxcms.com",
 "townnews-staging.com",
 "tuxfamily.org",
+"dd-dns.de",
+"diskstation.eu",
+"diskstation.org",
+"dray-dns.de",
+"draydns.de",
+"dyn-vpn.de",
+"dynvpn.de",
+"mein-vigor.de",
+"my-vigor.de",
+"my-wan.de",
+"syno-ds.de",
+"synology-diskstation.de",
+"synology-ds.de",
 "hk.com",
 "hk.org",
 "ltd.hk",
 "inc.hk",
+"lib.de.us",
 "router.management",
+"remotewd.com",
+"wmflabs.org",
 "yolasite.com",
+"ybo.faith",
+"yombo.me",
+"homelink.one",
+"ybo.party",
+"ybo.review",
+"ybo.science",
+"ybo.trade",
 "za.net",
 "za.org",
+"now.sh",
+"cc.ua",
+"inf.ua",
+"ltd.ua",
 "co.za",
 "co.il",
 0};
