@@ -19,19 +19,16 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "rec_channel.hh"
 #include <iostream>
 #include "pdnsexception.hh"
 #include "arguments.hh"
-#include "config.h"
 
 #include "namespaces.hh"
 
-#ifndef RECURSOR
-#include "statbag.hh"
-StatBag S;
-#endif
-
 ArgvMap &arg()
 {
   static ArgvMap arg;
@@ -42,23 +39,25 @@ static void initArguments(int argc, char** argv)
 {
   arg().set("config-dir","Location of configuration directory (recursor.conf)")=SYSCONFDIR;
 
-  arg().set("socket-dir","Where the controlsocket will live")=LOCALSTATEDIR;
+  arg().set("socket-dir",string("Where the controlsocket will live, ")+LOCALSTATEDIR+" when unset and not chrooted" )="";
+  arg().set("chroot","switch to chroot jail")="";
   arg().set("process","When controlling multiple recursors, the target process number")="";
   arg().set("timeout", "Number of seconds to wait for the recursor to respond")="5";
   arg().set("config-name","Name of this virtual configuration - will rename the binary image")="";
   arg().setCmd("help","Provide this helpful message");
+  arg().setCmd("version","Show the version of this program");
 
   arg().laxParse(argc,argv);  
-  if(arg().mustDo("help")) {
+  if(arg().mustDo("help") || arg().getCommands().empty()) {
     cout<<"syntax: rec_control [options] command, options as below: "<<endl<<endl;
     cout<<arg().helpstring(arg()["help"])<<endl;
-    exit(0);
+    cout<<"In addition, 'rec_control help' can be used to retrieve a list\nof available commands from PowerDNS"<<endl;
+    exit(arg().mustDo("help") ? 0 : 99);
   }
 
-  if(arg().getCommands().empty()) {
-    cerr<<"syntax: rec_control [options] command, options as below: "<<endl<<endl;
-    cerr<<arg().helpstring(arg()["help"])<<endl;
-    exit(99);
+  if(arg().mustDo("version")) {
+    cout<<"rec_control version "<<VERSION<<endl;
+    exit(0);
   }
 
   string configname=::arg()["config-dir"]+"/recursor.conf";
@@ -67,9 +66,21 @@ static void initArguments(int argc, char** argv)
   
   cleanSlashes(configname);
 
-  if(!::arg().preParseFile(configname.c_str(), "socket-dir", LOCALSTATEDIR)) 
+  if(!::arg().preParseFile(configname.c_str(), "socket-dir", ""))
     cerr<<"Warning: unable to parse configuration file '"<<configname<<"'"<<endl;
+  if(!::arg().preParseFile(configname.c_str(), "chroot", ""))
+    cerr<<"Warning: unable to parse configuration file '"<<configname<<"'"<<endl;
+
   arg().laxParse(argc,argv);   // make sure the commandline wins
+
+  if (::arg()["socket-dir"].empty()) {
+    if (::arg()["chroot"].empty())
+      ::arg().set("socket-dir") = LOCALSTATEDIR;
+    else
+      ::arg().set("socket-dir") = ::arg()["chroot"] + "/";
+  } else if (!::arg()["chroot"].empty()) {
+    ::arg().set("socket-dir") = ::arg()["chroot"] + "/" + ::arg()["socket-dir"];
+  }
 }
 
 int main(int argc, char** argv)
@@ -98,6 +109,10 @@ try
   }
   rccS.send(command);
   string receive=rccS.recv(0, arg().asNum("timeout"));
+  if(receive.compare(0, 7, "Unknown") == 0) {
+    cerr<<receive<<endl;
+    return 1;
+  }
   cout<<receive;
   return 0;
 }
